/**
 * The material property node.
 * <p>
 * The material is an (r,g,b,a) value.
 *
 * @version 	1.00
 * @author      Blair MacIntyre
 */
public class jooglMaterial extends jooglProperty {
    /** the material value, used for the cache if the property is dynamic. */
    protected float[] mat = new float[4];
    
    /** 
     * Create a material with color (r,g,b,1)
     *
     * @param r		the red component of the material
     * @param g		the green component of the material
     * @param b		the blue component of the material
     */
    public jooglMaterial(float r, float g, float b)
    {
        super(PROP_MATERIAL);
        mat[0] = r;
        mat[1] = g;
        mat[2] = b;
        mat[3] = 1.0f;
    }
    
    /** 
     * Create a material with color (r,g,b,a)
     *
     * @param r		the red component of the material
     * @param g		the green component of the material
     * @param b		the blue component of the material
     * @param a		the alpha component of the material
     */
    public jooglMaterial(float r, float g, float b, float a)
    {
        super(PROP_MATERIAL);
        mat[0] = r;
        mat[1] = g;
        mat[2] = b;
        mat[3] = a;
    }
    
    /** 
     * Create a material with color (r,g,b,1)
     *
     * @param r		the red component of the material
     * @param g		the green component of the material
     * @param b		the blue component of the material
     */
    public jooglMaterial(double r, double g, double b)
    {
        super(PROP_MATERIAL);
        mat[0] = (float)r;
        mat[1] = (float)g;
        mat[2] = (float)b;
        mat[3] = 1.0f;
    }
    
    /** 
     * Create a material with color (r,g,b,a)
     *
     * @param r		the red component of the material
     * @param g		the green component of the material
     * @param b		the blue component of the material
     * @param a		the alpha component of the material
     */
    public jooglMaterial(double r, double g, double b, double a)
    {
        super(PROP_MATERIAL);
        mat[0] = (float)r;
        mat[1] = (float)g;
        mat[2] = (float)b;
        mat[3] = (float)a;
    }
    
    /** 
     * Create a material with color (0.5,0.5,0.5,1)
     */
    public jooglMaterial()
    {
        super(PROP_MATERIAL);
        mat[0] = 0.5f;
        mat[1] = 0.5f;
        mat[2] = 0.5f;
        mat[3] = 1.0f;
    }
    
    /** 
     * Set the material color to (r,g,b,1)
     *
     * @param r		the red component of the material
     * @param g		the green component of the material
     * @param b		the blue component of the material
     */
    public void setColor(float r, float g, float b)
    {
        mat[0] = r;
        mat[1] = g;
        mat[2] = b;
        mat[3] = 1.0f;
        //redrawWindows();
    }

    /** 
     * Set the material color to (r,g,b,a)
     *
     * @param r		the red component of the material
     * @param g		the green component of the material
     * @param b		the blue component of the material
     * @param a		the alpha component of the material
     */
    public void setColor(float r, float g, float b, float a)
    {
        mat[0] = r;
        mat[1] = g;
        mat[2] = b;
        mat[3] = a;
        //redrawWindows();
    }

    /** 
     * Get the material color (r,g,b,a)
     *
     * @return a 4 element array containing the material color
     */
    public float[] getColor()
    {
        float[] m = new float[4];
        m[0] = mat[0];
        m[1] = mat[1];
        m[2] = mat[2];
        m[3] = mat[3];
        return m;
    }
    
    /**
     * Test to see if the color is transparent
     *
     * @return true if the material is not opaque
     */
    public boolean isTransparent()
    {
        return (mat[3] < 0.9999999);
    }

    /** 
     * Get the value of the property at time t.
     *
     * @param time the time in milliseconds at which the property should be evaluated
     * @return the draw mode
     */
    public float[] getValue(long time)
    {
        float[] m = new float[4];
        
        // this is a constant material.  Override this method to create a time varying property.
        m[0] = mat[0];
        m[1] = mat[1];
        m[2] = mat[2];
        m[3] = mat[3];
        
        return m;
    }

    /**
     * Internal Method, to cache the value of the property at time t in the property state variable.
     *
     * @param time the time in milliseconds at which the property should be evaluated
     */
    protected void cacheValue(long time)
    {
        // constant, do nothing
    }
    
    /** Internal Method, to set the OpenGL state for this property.  Called when the stacks are pushed and popped.
     * <p>
     * Only put things here that can be called multiple times safely. 
     */
    protected void apply()
    {
        //gl.glMaterialfv (GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat);
        gl.glColor4f(mat[0], mat[1], mat[2], mat[3]);
    }
}
