/**
 * The root node that associates a scene graph with a camera and viewport to render it in.
 * <p>
 * The root node renders a scene graph into a viewport.  The viewport position and size can be absolute values,
 * specified in pixels, or a relative values, specified in floating point values from 0.0 to 1.0.  If relative
 * values are used, the viewport will resize when the window changes shape.
 * <p>
 * The background of the viewport can be any desired color.
 *
 * @version 	1.00
 * @author      Blair MacIntyre
 */
public class jooglRoot extends jooglGroup {
    /** The camera for this viewport. */
    protected jooglCamera cam = new jooglPerspectiveCamera();
    
    /** The red component of the background color for this viewport; default is dark grey. */
    protected float bgRed = 0.05f;
    /** The green component of the background color for this viewport; default is dark grey. */
    protected float bgGreen = 0.05f;
    /** The blue component of the background color for this viewport; default is dark grey. */
    protected float bgBlue = 0.05f;
    /** The alpha component of the background color for this viewport; default is dark grey. */
    protected float bgAlpha = 1.0f;
    
    /** The absolute X position of the window, either specified or computed. */
    protected int abs_x = 0;
    /** The absolute Y position of the window, either specified or computed. */
    protected int abs_y = 0;
    /** The absolute width of the window, either specified or computed. */
    protected int abs_width = 1;
    /** The absolute height of the window, either specified or computed. */
    protected int abs_height = 1;
    
    /** The relative X position of the window, in the range 0.0 to 1.0 */
    protected float rel_x = 0.0f;
    /** The relative Y position of the window, in the range 0.0 to 1.0 */
    protected float rel_y = 0.0f;
    /** The relative width of the window, in the range 0.0 to 1.0 */
    protected float rel_width = 1.0f;
    /** The relative height of the window, in the range 0.0 to 1.0 */
    protected float rel_height = 1.0f;
    
    /** The current computed aspect ratio of the viewport. */
    protected double aspect = 1.0;

    /** Is this viewport specified in absolute or relative values? */
    protected boolean absolute = false;
    
    /** The default constructor. */
    public jooglRoot() 
    {
        super();
    }

    /** A constructor that specifies an absolute position and size.
     *
     * @param x the absolute x value
     * @param y the absolute y value
     * @param width the absolute width
     * @param heigth the absolute height
     */
    public jooglRoot(int x, int y, int width, int height) 
    {
        super();
        abs_x = x;
        abs_y = y;
        abs_width = width;
        abs_height = height;
        absolute = true;
    }

    /** A constructor that specifies a relative position and size.
     *
     * @param x the relative x value
     * @param y the relative y value
     * @param width the relative width
     * @param heigth the relative height
     */
    public jooglRoot(float x, float y, float width, float height)
    {
        super();
        rel_x = x;
        rel_y = y;
        rel_width = width;
        rel_height = height;
        absolute = false;
    }

    /** Set the camera to use to render this scene graph into this viewport.
     * 
     * @param c the new camera object.
     */
    public void setCamera(jooglCamera c)
    {
        cam = c;
        // cam.setAspect(aspect);
        redrawWindows();
    }
        
    /** Get the current camera for this root.
     *
     * @return the current camera object.
     */
    public jooglCamera getCamera()
    {
        return cam;
    }
        
    /**
     * Set the background color.
     * 
     * @param r		the red component of the background
     * @param g		the green component of the background
     * @param b		the blue component of the background
     */
    public void setColor(float r, float g, float b)
    {
        bgRed = r;
        bgGreen = g;
        bgBlue = b;
        bgAlpha = 1.0f;
        redrawWindows();
    }
    
    /**
     * Set the background color, with alpha.
     * 
     * @param r		the red component of the background
     * @param g		the green component of the background
     * @param b		the blue component of the background
     */
    public void setColor(float r, float g, float b, float a)
    {
        bgRed = r;
        bgGreen = g;
        bgBlue = b;
        bgAlpha = a;
        redrawWindows();
    }
    
    /**
     * void adjustToWindowReshape(int w, int h)
     *
     * Called after the first paint command and any subsequent resize.  Recompute the absolute viewport
     * position and size from the relative values if necessary.
     *
     * @param w the width
     * @param h the height
     */
    public void adjustToWindowReshape(int w, int h)
    {
        if (absolute)
            return; // do nothing
            
        abs_x = (int)(((float)w) * rel_x);
        abs_y = (int)(((float)h) * rel_y);
        abs_height = (int)(((float)h) * rel_height);
        abs_width = (int)(((float)w) * rel_width);
        aspect = (double)abs_width/(double)abs_height;
        //System.out.println("w=" + w + ", h=" + h + ", ax=" + abs_x + ", ay = " 
        //        + abs_y + ", aw=" + abs_width + ", ah=" + abs_height 
        //        + ", aspect=" + aspect);
        //cam.setAspect(aspect);
    }

    /** Internal Method, to set the OpenGL viewport for this root.
     * <p>
     * Sets the viewport to correspond to the viewport for this root.
     */
    protected void setViewport()
    {
        // set the viewport
        //System.out.println(" viewport(ax=" + abs_x + ", ay = " 
        //        + abs_y + ", aw=" + abs_width + ", ah=" + abs_height 
       //         + ")");
        gl.glViewport(abs_x, abs_y, abs_width, abs_height);
    }
}
